/*
   Routines to Access the Romulator - SNES/GENESIS PC Programming Interface

   As far as I know these work for the SNES part, I don't own a
   Say-Gah (bubsy!)

   Thanks to Carl at the Busop Archives for his sources. His sources
   pointed me into a decent direction in writing this thing.

   I wish to point out that the Romulator is a great card!  And I have
   been able to run many things that don't need to be patched.
   (ie Super Punch Out, and a couple of others).  But the guys at
   Rhotech Labs wouldn't send me the manual to the card (even offer
   to pay for it, since it has the programming info in there).
   But no prob, hacking around in that crappy send utility that
   they bundle with the Romulator, I figured most of the things out.

   In the last version, I didn't explain Write Back register very well.

   Write Back register allows you to decide whether allow the game
   to write back to itself.  Some games use SRAM check as a method
   of checking to see if this is a copied game. (various methods
   have been applied and all lead to the same conclusion is this
   a copier?!?) if so game doesn't play boohoo.  You either
   patch the game or you just simpling pull 2x8h register and
   protect the memory.

   Now to use it you need to reset the Romulator in this method.

   Stop Romulator        - 2xEh
   Write Back On         - 2x6h
   Reset Address Latches - 2xAh

   We always turn Write Back ON when when uploading a binary to
   the card.  If we did turn it off, WE CAN'T WRITE TO IT AT ALL!

   Poke your game into memory using registers

   1st byte - 2x2h (all even address???)
   2nd byte - 2x3h (all odd address????)

   I have noticed the romulator isn't picky about size, so that's
   nice =)

   Now if the game doesn't use sram checks, just leave it alone
   from here.  If the game does do a sram check, then
   just in 2x8h, and the memory is protected.

   How do you know if a game use sram Checks??? well you
   can either scan for something like lda $700000, or just
   try to play it and it either all messed up or it says
   'Copying Video Games is illegal, please call the police
    and have yourself arrested'
   
  after that pull in 2xCh to run the binary

   But if you think I made mistake, lemme know! or if you want to
   send me the programming part of the romulator manual *PLEASE DO*

   No Thanks, Credits or Greets to the Guys at Rhotech Labs, for
   they did not wish to send me a manual with the Romulator.

   Some ideas came from Carl at the Busop Archives

   This is no means complete, if you want the more complete version
   of the source that backups karts and such look for me in #snes

   Mind Rape
*/

/*
    2x2 - Load Low  byte       RW
    2x3 - Load High Byte       RW

    2x6 - Write Back On        R    do allow write backs
    2x8 - Write Back Off       R    don't allow write backs

    2xA - Reset Address Latch  R

    2xC - Run Binary           R

    2xE - Stops Romulator      R

 */

void Init_Romulator(unsigned int port)
{
  asm {
        mov     dx,[port]
        or      dx,0eh                  /* Reset Game Machine  2xEh */
        in      al,dx

        sub     dx,8                    /* allow writes to the memory */
        in      al,dx                   /* 2x6h                       */

        add     dx,4                    /* Reset Address latches 2xAh */
        in      al,dx                   /* to the start of the card's */
      }                                 /* memory                     */
}

void Run_Romulator(unsigned int port)
{
 asm {
       mov dx,[port]
       or dx,0ch          /* Run the Game Image 2xCh */
       in al,dx
     }
}


void UploadData(char buffer[],unsigned int size,unsigned int port)
{ int i;
  unsigned int low_port;

  char Low_byte;
  char High_byte;

  low_port = port + 0x02;

  for (i=0;i<size;i+=2)
    {
      Low_byte = buffer[i];
      High_byte = buffer[i+1];

      asm {
            mov dx,low_port
            mov al,Low_byte
            mov ah,High_byte
            out dx,ax
          }
   }
}


void SetWriteBack(unsigned int write,unsigned int port)
{
  asm {
        mov dx,[port]
        or  dx, write
        in al,dx
      }
}

