#include <stdio.H>
#include <stdlib.h>
#include <string.h>
#include "rom.h"

void Help(void);
void Upload2Card(FILE *source);
void Detect_Image(FILE *source);
void getops(int argc,char *argv[]);

char title[] = {"Send - Version 1.0 - Romulator SNES Console Utility\n"
                 "(C) 1994 Damaged Cybernetics, All Rights Reserved\n"
               };

unsigned int writeback = 6;      /* allow writebacks by default   */
unsigned char header = 0;        /* User Forceable Header Skip    */
unsigned char filename[15];      /* store the filename here       */
unsigned int port = 0x230;       /* base port for Romulator       */

unsigned long filesize;
FILE *source;

void Upload2Card(FILE *source)
{ unsigned char buffer[32768];
  int size;
  char temp;

  while (!feof(source))
    {
      size = fread(buffer,1,32768,source);
      UploadData(buffer,size,port);
    }

  fcloseall();
}

void Detect_Image(FILE *source)
{  unsigned char buf[512];

  rewind(source);                /* goto start of image  */

  fseek(source,0L,SEEK_END);
  filesize = ftell(source);      /* snag the file size */
  rewind(source);                /* rewind             */

  switch (header)
    {
      case 1 : puts("[User] forced Header Skipping");
               fseek(source,512,SEEK_SET);
               return;

      case 2 : puts("User Forced MGD Upload");
               return;
    }

  fread(buf,512,1,source);       /* most headers are 512 */
  /* a note to remember                                  */
  /* if we determine that this image is not a raw binary */
  /* we already skipped the header!!!                    */

  if (buf[8] == 0xAA && buf[9] == 0xBB)  /* check for SMC/SWC */
    {
      puts("Uploading SMC Image - Skipping Header");
      return;
    }

  if (buf[4] == 0x77 && buf[5] == 0x83)  /* check for FIG */
    {
      puts("Uploading FIG Image - Skipping Header");
      return;
    }

 rewind(source);                /* no header that we know of, rewind */
 puts("Uploading MGD Image");
}

void Help()
{
  puts("usage: send [options] <image>  (ie. send lionking.snk /r)\n"
       "Options:\n"
       "  S  - Force 512 byte header skipping\n"
       "  R  - Force MGD (raw binary) upload\n"
       "  W  - Writebacks - /W1 = On; /W2 = Off\n"
       "  Px - Port to upload data to Romulator (ie, /Pb = 2b0h)\n"
       "       default port = 230h\n"
      );
}

void getops(int argc,char *argv[])
{ int i;

  for (i=1;i<argc;i++)
    if (argv[i][0] == '/' || argv[i][0] == '-')
      {
        switch (argv[i][1])
          {
            case 's':
            case 'S': header = 1; break; /* skip header */

            case 'r':
            case 'R': header = 2; break; /* force MGD Upload */

            case 'w':
            case 'W': switch(argv[i][2])
                        {
                          case '1' : writeback = 6; break; /* write back on */
                          case '2' : writeback = 8; break; /* off           */
                          default  : puts("%%error: invalid write back def");
                                     exit(1);
                        }
                       break;

            case 'p':
            case 'P': switch (argv[i][2])
                        {
                          case 'b':
                          case 'B': port = 0x2b0; break;
                          case '7': port = 0x270; break;
                          case '3': port = 0x230; break;
                          default: puts("%%error: invalid port def");
                                   exit(1);
                        }
                      break;

            case '?': Help(); exit(0); break;
          }
       }
     else
       strcpy(filename,argv[i]);
}

void main(int argc,char *argv[])
{ unsigned int i;

  puts(title);

  if (argc < 3)
    {
      Help();
      exit(0);
    }

  header = 0;   /* no header on this file as default */

  getops(argc,argv);

  if ((source = fopen(filename,"rb")) == NULL)
    {
      printf("Cannot open input file: %s\n",filename);
      exit(1);
    }

  Detect_Image(source);

  printf("Image Size     : %ld bytes\n",filesize);
  printf("Using Base Port: %Xh\n",port);

  printf("Writebacks are : ");
  if (writeback == 6)
    puts("ON");
  else
    puts("OFF");

  Init_Romulator(port);
  Upload2Card(source);

  SetWriteBack(writeback,port); /* this is only needed for writeback off  */
                                /* not worth putting a check in for since */

  Run_Romulator(port);

  puts("Finished!");
}
